@@ -94,7 +94,7 @@ class ConsoleState:
 
 
 class ConsoleApplication(object):
-    def __init__(self, run_until_return=await_enter, on_stop=None):
+    def __init__(self, run_until_return=await_enter, on_stop=None, args=None):
         plain_terminal = os.environ.get("TERM", "").lower() == "none"
 
         # Windows doesn't have SIGPIPE
@@ -103,43 +103,8 @@ class ConsoleApplication(object):
 
         colorama.init(strip=True if plain_terminal else None)
 
-        parser = argparse.ArgumentParser(usage=self._usage())
-
-        if self._needs_device():
-            relays = []
-
-            parser.add_argument("-D", "--device", help="connect to device with the given ID", metavar="ID", dest="device_id")
-            parser.add_argument("-U", "--usb", help="connect to USB device", action='store_const', const='usb', dest="device_type")
-            parser.add_argument("-R", "--remote", help="connect to remote frida-server", action='store_const', const='remote', dest="device_type")
-            parser.add_argument("-H", "--host", help="connect to remote frida-server on HOST")
-            parser.add_argument("--certificate", help="speak TLS with HOST, expecting CERTIFICATE")
-            parser.add_argument("--origin", help="connect to remote server with “Origin” header set to ORIGIN")
-            parser.add_argument("--token", help="authenticate with HOST using TOKEN")
-            parser.add_argument("--keepalive-interval", help="set keepalive interval in seconds, or 0 to disable (defaults to -1 to auto-select based on transport)", metavar="INTERVAL", type=int)
-            parser.add_argument("--p2p", help="establish a peer-to-peer connection with target", action='store_const', const='p2p', dest="session_transport", default="multiplexed")
-            parser.add_argument("--stun-server", help="set STUN server ADDRESS to use with --p2p", metavar="ADDRESS")
-            parser.add_argument("--relay", help="add relay to use with --p2p", metavar="address,username,password,turn-{udp,tcp,tls}", action='append', type=deserialize_relay)
-
-        if self._needs_target():
-            parser.add_argument("-f", "--file", help="spawn FILE", dest="target", type=create_target_parser("file"))
-            parser.add_argument("-F", "--attach-frontmost", help="attach to frontmost application", dest="target", type=create_target_parser("frontmost"))
-            parser.add_argument("-n", "--attach-name", help="attach to NAME", metavar="NAME", dest="target", type=create_target_parser("name"))
-            parser.add_argument("-p", "--attach-pid", help="attach to PID", metavar="PID", dest="target", type=create_target_parser("pid"))
-            parser.add_argument("-W", "--await", help="await spawn matching PATTERN", metavar="PATTERN", dest="target", type=create_target_parser("gated"))
-            parser.add_argument("--stdio", help="stdio behavior when spawning (defaults to “inherit”)", choices=["inherit", "pipe"], default="inherit")
-            parser.add_argument("--aux", help="set aux option when spawning, such as “uid=(int)42” (supported types are: string, bool, int)", metavar="option", action="append", dest="aux", default=[])
-            parser.add_argument("--realm", help="realm to attach in", choices=["native", "emulated"], default="native")
-            parser.add_argument("--runtime", help="script runtime to use", choices=["qjs", "v8"])
-            parser.add_argument("--debug", help="enable the Node.js compatible script debugger", action="store_true", dest="enable_debugger", default=False)
-            parser.add_argument("--squelch-crash", help="if enabled, will not dump crash report to console", action="store_true", default=False)
-            parser.add_argument("args", help="extra arguments and/or target", nargs="*")
-
-        parser.add_argument("-O", "--options-file", help="text file containing additional command line options", metavar="FILE")
-        parser.add_argument('--version', action='version', version=frida.__version__)
-
-        self._add_options(parser)
-
-        real_args = compute_real_args(parser)
+        parser = self._initialize_arguments_parser()
+        real_args = compute_real_args(parser, args=args)
         options = parser.parse_args(real_args)
 
         # handle scripts that don't need a target
@@ -160,7 +125,7 @@ class ConsoleApplication(object):
             self._keepalive_interval = options.keepalive_interval
             self._session_transport = options.session_transport
             self._stun_server = options.stun_server
-            self._relays = relays
+            self._relays = options.relays
         else:
             self._device_id = None
             self._device_type = None
@@ -227,6 +192,52 @@ class ConsoleApplication(object):
         except Exception as e:
             parser.error(str(e))
 
+    def _initialize_arguments_parser(self):
+        parser = self._initialize_base_arguments_parser()
+        self._add_options(parser)
+        return parser
+
+    def _initialize_base_arguments_parser(self):
+        parser = argparse.ArgumentParser(usage=self._usage())
+
+        if self._needs_device():
+            self._add_device_arguments(parser)
+
+        if self._needs_target():
+            self._add_target_arguments(parser)
+
+        parser.add_argument("-O", "--options-file", help="text file containing additional command line options", metavar="FILE")
+        parser.add_argument('--version', action='version', version=frida.__version__)
+
+        return parser
+
+    def _add_device_arguments(self, parser):
+        parser.add_argument("-D", "--device", help="connect to device with the given ID", metavar="ID", dest="device_id")
+        parser.add_argument("-U", "--usb", help="connect to USB device", action='store_const', const='usb', dest="device_type")
+        parser.add_argument("-R", "--remote", help="connect to remote frida-server", action='store_const', const='remote', dest="device_type")
+        parser.add_argument("-H", "--host", help="connect to remote frida-server on HOST")
+        parser.add_argument("--certificate", help="speak TLS with HOST, expecting CERTIFICATE")
+        parser.add_argument("--origin", help="connect to remote server with “Origin” header set to ORIGIN")
+        parser.add_argument("--token", help="authenticate with HOST using TOKEN")
+        parser.add_argument("--keepalive-interval", help="set keepalive interval in seconds, or 0 to disable (defaults to -1 to auto-select based on transport)", metavar="INTERVAL", type=int)
+        parser.add_argument("--p2p", help="establish a peer-to-peer connection with target", action='store_const', const='p2p', dest="session_transport", default="multiplexed")
+        parser.add_argument("--stun-server", help="set STUN server ADDRESS to use with --p2p", metavar="ADDRESS")
+        parser.add_argument("--relay", help="add relay to use with --p2p", metavar="address,username,password,turn-{udp,tcp,tls}", dest="relays", action='append', type=deserialize_relay)
+
+    def _add_target_arguments(self, parser):
+        parser.add_argument("-f", "--file", help="spawn FILE", dest="target", type=create_target_parser("file"))
+        parser.add_argument("-F", "--attach-frontmost", help="attach to frontmost application", dest="target", action="store_const", const=('frontmost', None))
+        parser.add_argument("-n", "--attach-name", help="attach to NAME", metavar="NAME", dest="target", type=create_target_parser("name"))
+        parser.add_argument("-p", "--attach-pid", help="attach to PID", metavar="PID", dest="target", type=create_target_parser("pid"))
+        parser.add_argument("-W", "--await", help="await spawn matching PATTERN", metavar="PATTERN", dest="target", type=create_target_parser("gated"))
+        parser.add_argument("--stdio", help="stdio behavior when spawning (defaults to “inherit”)", choices=["inherit", "pipe"], default="inherit")
+        parser.add_argument("--aux", help="set aux option when spawning, such as “uid=(int)42” (supported types are: string, bool, int)", metavar="option", action="append", dest="aux", default=[])
+        parser.add_argument("--realm", help="realm to attach in", choices=["native", "emulated"], default="native")
+        parser.add_argument("--runtime", help="script runtime to use", choices=["qjs", "v8"])
+        parser.add_argument("--debug", help="enable the Node.js compatible script debugger", action="store_true", dest="enable_debugger", default=False)
+        parser.add_argument("--squelch-crash", help="if enabled, will not dump crash report to console", action="store_true", default=False)
+        parser.add_argument("args", help="extra arguments and/or target", nargs="*")
+
     def run(self):
         mgr = frida.get_device_manager()
 
@@ -637,8 +648,10 @@ class ConsoleApplication(object):
         return state_dir
 
 
-def compute_real_args(parser):
-    real_args = normalize_options_file_args(sys.argv[1:])
+def compute_real_args(parser, args=None):
+    if args is None:
+        args = sys.argv[1:]
+    real_args = normalize_options_file_args(args)
 
     files_processed = set()
     while True:
