@@ -2,9 +2,12 @@
 from __future__ import unicode_literals, print_function
 
 import os
+import shlex
 import signal
 import string
 import threading
+import sys
+import time
 
 
 def main():
@@ -14,7 +17,6 @@ def main():
     import os
     import platform
     import re
-    import sys
     try:
         from urllib.request import build_opener
     except:
@@ -22,6 +24,7 @@ def main():
 
     from colorama import Fore, Style
     import frida
+    from prompt_toolkit.shortcuts import prompt
     from prompt_toolkit import PromptSession
     from prompt_toolkit.history import FileHistory
     from prompt_toolkit.completion import Completion, Completer
@@ -45,6 +48,8 @@ def main():
             self._monitored_files = {}
             self._autoperform = False
             self._autoperform_option = False
+            self._autoreload = True
+            self._quiet_start = None
 
             super(REPLApplication, self).__init__(self._process_input, self._on_stop)
 
@@ -77,11 +82,14 @@ def main():
             parser.add_argument("-c", "--codeshare", help="load CODESHARE_URI", metavar="CODESHARE_URI", dest="codeshare_uri")
             parser.add_argument("-e", "--eval", help="evaluate CODE", metavar="CODE", action='append', dest="eval_items")
             parser.add_argument("-q", help="quiet mode (no prompt) and quit after -l and -e", action='store_true', dest="quiet", default=False)
+            parser.add_argument("-t", "--timeout", help="seconds to wait before terminating in quiet mode", dest="timeout", default=0)
             parser.add_argument("--no-pause", help="automatically start main thread after startup", action='store_true', dest="no_pause", default=False)
             parser.add_argument("-o", "--output", help="output to log file", dest="logfile")
             parser.add_argument("--eternalize", help="eternalize the script before exit", action='store_true', dest="eternalize", default=False)
             parser.add_argument("--exit-on-error", help="exit with code 1 after encountering any exception in the SCRIPT", action='store_true', dest="exit_on_error", default=False)
             parser.add_argument("--auto-perform", help="wrap entered code with Java.perform", action='store_true', dest="autoperform", default=False)
+            parser.add_argument("--auto-reload", help="Enable auto reload of provided scripts and c module (on by default, will be required in the future)", action='store_true', dest="autoreload", default=True)
+            parser.add_argument("--no-auto-reload", help="Disable auto reload of provided scripts and c module", action='store_false', dest="autoreload", default=True)
 
         def _initialize(self, parser, options, args):
             self._user_scripts = list(map(os.path.abspath, options.user_scripts))
@@ -102,7 +110,7 @@ def main():
 
             if options.user_cmodule is not None:
                 self._user_cmodule = os.path.abspath(options.user_cmodule)
-                with open(self._user_cmodule, 'rb') as f:
+                with open(self._user_cmodule, 'rb'):
                     pass
             else:
                 self._user_cmodule = None
@@ -114,10 +122,12 @@ def main():
             self._pending_eval = options.eval_items
 
             self._quiet = options.quiet
+            self._quiet_timeout = float(options.timeout)
             self._no_pause = options.no_pause
             self._eternalize = options.eternalize
             self._exit_on_error = options.exit_on_error
             self._autoperform_option = options.autoperform
+            self._autoreload = options.autoreload
 
             if options.logfile is not None:
                 self._logfile = codecs.open(options.logfile, 'w', 'utf-8')
@@ -181,7 +191,7 @@ def main():
             else:
                 self._unload_script()
 
-            with frida.Cancellable() as c:
+            with frida.Cancellable():
                 self._demonitor_all()
 
             if self._logfile is not None:
@@ -191,7 +201,8 @@ def main():
                 self._print("\nThank you for using Frida!")
 
         def _load_script(self):
-            self._monitor_all()
+            if self._autoreload:
+                self._monitor_all()
 
             if len(self._user_scripts) == 0:
                 name = "repl"
@@ -299,6 +310,20 @@ def main():
                             self._pending_eval = None
                     else:
                         if self._quiet:
+                            if self._quiet_timeout > 0:
+                                if self._quiet_start is None:
+                                    self._quiet_start = time.time()
+                                passed_time = time.time() - self._quiet_start
+                                while self._quiet_timeout > passed_time and reactor.is_running():
+                                    sleep_time = min(1, self._quiet_timeout - passed_time)
+                                    if self._stopping.wait(sleep_time):
+                                        break
+                                    if self._dumb_stdin_reader is not None:
+                                        with self._dumb_stdin_reader._lock:
+                                            if self._dumb_stdin_reader._saw_sigint:
+                                                break
+                                    passed_time = time.time() - self._quiet_start
+
                             self._exit_status = 0 if self._errors == 0 else 1
                             return
 
@@ -349,6 +374,27 @@ def main():
                     except frida.OperationCancelledError:
                         return
 
+        def _get_confirmation(self, question, default_answer = False):
+            if default_answer:
+                prompt_string = question + " [Y/n] "
+            else:
+                prompt_string = question + " [y/N] "
+
+            if self._have_terminal and not self._plain_terminal:
+                answer = prompt(prompt_string)
+            else:
+                answer = self._dumb_stdin_reader.read_line(prompt_string)
+                self._print(answer)
+
+            if answer.lower() not in ("y", "yes", "n", "no", ""):
+                return self._get_confirmation(question, default_answer=default_answer)
+
+            if default_answer:
+                return answer.lower() != "n" and answer.lower() != "no"
+
+            return answer.lower() == "y" or answer.lower() == "yes"
+
+
         def _eval_and_print(self, expression):
             success = False
             try:
@@ -428,16 +474,18 @@ def main():
         # Negative means at least abs(val) - 1
         _magic_command_args = {
             'resume': _repl_magic.Resume(),
+            'load': _repl_magic.Load(),
             'reload': _repl_magic.Reload(),
             'unload': _repl_magic.Unload(),
             'autoperform': _repl_magic.Autoperform(),
+            'autoreload': _repl_magic.Autoreload(),
             'exec': _repl_magic.Exec(),
             'time': _repl_magic.Time(),
             'help': _repl_magic.Help()
         }
 
         def _do_magic(self, statement):
-            tokens = statement.split(" ")
+            tokens = shlex.split(statement)
             command = tokens[0]
             args = tokens[1:]
 
@@ -478,7 +526,10 @@ def main():
         def _is_java_available(self):
             script = self._session.create_script(name="java_check", source="rpc.exports.javaAvailable = () => Java.available;", runtime=self._runtime)
             script.load()
-            return script.exports.java_available()
+            try:
+                return script.exports.java_available()
+            except:
+                return False
 
         def _refresh_prompt(self):
             self._prompt_string = self._create_prompt()
@@ -705,22 +756,16 @@ URL: {url}
                 url="https://codeshare.frida.re/@{}".format(uri)
             ))
 
-            while True:
-                prompt_string = "Are you sure you'd like to trust this project? [y/N] "
-                response = get_input(prompt_string)
-
-                if response.lower() in ('n', 'no') or response == '':
-                    return None
-
-                if response.lower() in ('y', 'yes'):
-                    self._print(
-                        "Adding fingerprint {} to the trust store! You won't be prompted again unless the code changes.".format(
-                            fingerprint))
-                    script = response_json['source']
-                    self._update_truststore({
-                        uri: fingerprint
-                    })
-                    return script
+            answer = self._get_confirmation("Are you sure you'd like to trust this project?")
+            if answer:
+                self._print(
+                    "Adding fingerprint {} to the trust store! You won't be prompted again unless the code changes.".format(
+                        fingerprint))
+                script = response_json['source']
+                self._update_truststore({
+                    uri: fingerprint
+                })
+                return script
 
         def _update_truststore(self, record):
             trust_store = self._get_or_create_truststore()
@@ -750,7 +795,7 @@ URL: {url}
 
             return trust_store
 
-        def _get_truststore_file(self):
+        def _get_or_create_truststore_file(self):
             truststore_file = os.path.join(self._get_or_create_data_dir(), 'codeshare-truststore.json')
             if not os.path.isfile(truststore_file):
                 self._migrate_old_config_file('codeshare-truststore.json', truststore_file)
@@ -783,7 +828,8 @@ URL: {url}
             return False
 
         def _on_device_found(self):
-            self._print("""\
+            if not self._quiet:
+                self._print("""\
    . . . .
    . . . .   Connected to {device_name} (id={device_id})""".format(device_id=self._device.id, device_name=self._device.name))
 
@@ -949,11 +995,17 @@ class DumbStdinReader(object):
     def __init__(self, valid_until):
         self._valid_until = valid_until
 
+        self._saw_sigint = False
         self._prompt = None
         self._result = None
         self._lock = threading.Lock()
         self._cond = threading.Condition(self._lock)
 
+        try:
+            self._get_input = raw_input
+        except NameError:
+            self._get_input = input
+
         worker = threading.Thread(target=self._process_requests, name="stdin-reader")
         worker.daemon = True
         worker.start()
@@ -987,7 +1039,7 @@ class DumbStdinReader(object):
                 prompt = self._prompt
 
             try:
-                line = get_input(prompt)
+                line = self._get_input(prompt)
             except Exception as e:
                 line = None
                 error = e
@@ -999,6 +1051,7 @@ class DumbStdinReader(object):
 
     def _cancel_line(self):
         with self._lock:
+            self._saw_sigint = True
             self._prompt = None
             self._result = (None, KeyboardInterrupt())
             self._cond.notify()
@@ -1008,7 +1061,6 @@ if os.environ.get("TERM", "") == 'dumb':
     try:
         from collections import namedtuple
         from epc.client import EPCClient
-        import sys
     except ImportError:
         def start_completion_thread(repl, epc_port=None):
             # Do nothing when we cannot import the EPC module.
@@ -1087,16 +1139,6 @@ else:
         _, _ = repl, epc_port
 
 
-try:
-    input_impl = raw_input
-except NameError:
-    input_impl = input
-
-
-def get_input(prompt_string):
-    return input_impl(prompt_string)
-
-
 if __name__ == '__main__':
     try:
         main()
