@@ -7,8 +7,6 @@ import os
 import platform
 import re
 import subprocess
-import threading
-import time
 
 import frida
 
@@ -470,7 +468,7 @@ class Repository(object):
                                 "annotate_post": annotate_post
                             })
                         break
-                except Exception as e:
+                except Exception:
                     pass
 
             if decorate:
